/****************************************************************************
**
** Copyright (C) 2015 Digia Plc and/or its subsidiary(-ies).
** Contact: http://www.qt-project.org/legal
**
** This file is part of the Qt Quick Controls module of the Qt Toolkit.
**
** $QT_BEGIN_LICENSE:LGPL$
** Commercial License Usage
** Licensees holding valid commercial Qt licenses may use this file in
** accordance with the commercial license agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and Digia.  For licensing terms and
** conditions see http://qt.digia.com/licensing.  For further information
** use the contact form at http://qt.digia.com/contact-us.
**
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 2.1 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU Lesser General Public License version 2.1 requirements
** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
**
** In addition, as a special exception, Digia gives you certain additional
** rights.  These rights are described in the Digia Qt LGPL Exception
** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 3.0 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU General Public License version 3.0 requirements will be
** met: http://www.gnu.org/copyleft/gpl.html.
**
**
** $QT_END_LICENSE$
**
****************************************************************************/

import QtQuick 2.0
import QtTest 1.0
import QtWebEngine 1.0
import QtWebEngine.testsupport 1.0

TestWebEngineView {
    id: webEngineView

    property string messageFromAlertDialog: ""
    property int confirmCount: 0
    property int promptCount: 0

    testSupport: WebEngineTestSupport {
        onAlertDialog: {
            webEngineView.messageFromAlertDialog = dialog.message
            dialog.accept()
        }

        onConfirmDialog: {
            webEngineView.confirmCount += 1
            if (dialog.message == "ACCEPT")
                dialog.accept()
            else
                dialog.reject()
        }

        onPromptDialog: {
            webEngineView.promptCount += 1
            if (dialog.message == "REJECT")
                dialog.reject()
            else {
                var reversedDefaultValue = dialog.defaultValue.split("").reverse().join("")
                dialog.accept(reversedDefaultValue)
            }
        }
    }

    TestCase {
        id: test
        name: "WebEngineViewJavaScriptDialogs"

        function init() {
            webEngineView.messageFromAlertDialog = ""
            webEngineView.confirmCount = 0
            webEngineView.promptCount = 0
        }

        function test_alert() {
            webEngineView.url = Qt.resolvedUrl("alert.html")
            verify(webEngineView.waitForLoadSucceeded())
            compare(webEngineView.messageFromAlertDialog, "Hello Qt")
        }

        function test_confirm() {
            webEngineView.url = Qt.resolvedUrl("confirm.html")
            verify(webEngineView.waitForLoadSucceeded())
            compare(webEngineView.confirmCount, 2)
            compare(webEngineView.title, "ACCEPTED REJECTED")
        }

        function test_prompt() {
            webEngineView.url = Qt.resolvedUrl("prompt.html")
            verify(webEngineView.waitForLoadSucceeded())
            compare(webEngineView.promptCount, 2)
            compare(webEngineView.title, "tQ olleH")
        }
    }
}
